from pydantic import BaseModel
from nonebot.typing import overrides

from nonebot.adapters import Event as BaseEvent

from .config import BaseInfo
from .message import Message


class Event(BaseEvent):
    __type__ = "Event"

    @overrides(BaseEvent)
    def get_type(self) -> str:
        return self.__type__

    @overrides(BaseEvent)
    def get_event_name(self) -> str:
        return self.__type__

    @overrides(BaseEvent)
    def get_event_description(self) -> str:
        return str(self.dict())

    @overrides(BaseEvent)
    def get_message(self) -> Message:
        raise ValueError("Event has no message!")

    @overrides(BaseEvent)
    def get_user_id(self) -> str:
        raise ValueError("Event has no user_id!")

    @overrides(BaseEvent)
    def get_session_id(self) -> str:
        raise ValueError("Event has no session_id!")

    @overrides(BaseEvent)
    def is_tome(self) -> bool:
        """获取事件是否与机器人有关的方法。"""
        raise NotImplementedError


class MessageEvent(Event):
    __type__: str = "message"
    user_info: BaseInfo
    message: Message

    @overrides(Event)
    def get_user_id(self) -> str:
        if self.user_info:
            return self.user_info.user_id
        return ""

    @overrides(Event)
    def get_message(self) -> Message:
        return self.message

    @overrides(Event)
    def get_session_id(self) -> str:
        return self.user_info.user_id

    @overrides(Event)
    def is_tome(self) -> bool:
        return False

    @overrides(Event)
    def get_event_description(self) -> str:
        return f"{self.user_info.nickname} from {self.get_message()}"

    @overrides(Event)
    def get_plaintext(self) -> str:
        return self.get_message().extract_plain_text()



from typing import Any, Union

from nonebot.typing import overrides
from nonebot.message import handle_event

from nonebot.adapters import Adapter
from nonebot.adapters import Bot as BaseBot

from .config import BaseInfo
from .terminal import console_view
from .event import Event, MessageEvent
from .message import Message, MessageSegment


class Bot(BaseBot):
    @overrides(BaseBot)
    def __init__(self, adapter: "Adapter", bot_config: BaseInfo):
        super().__init__(adapter, "0")
        self.bot_config: BaseInfo = bot_config

    @property
    def type(self) -> str:
        return "Console"

    @overrides(BaseBot)
    async def send(
        self,
        event: Event,
        message: Union[str, Message, MessageSegment],
        **kwargs: Any,
    ) -> Any:
        if isinstance(event, MessageEvent):
            event.user_info = self.bot_config
            await console_view.send_message(event, message)

    async def handle_event(self, event: Event) -> None:
        """处理收到的事件。"""
        await handle_event(self, event)



from typing import Union

from rich import box
from rich.text import Text
from rich.panel import Panel
from rich.style import Style
from textual.widget import Widget
from textual.reactive import Reactive


class Input(Widget):
    height: Reactive[int] = Reactive(3)
    value: str = ""
    is_input: Reactive[bool] = Reactive(True)
    cursor_position: int = 0

    def __init__(self, name: Union[str, None] = None, placeholder: str = "") -> None:
        super().__init__(name)
        self.placeholder: str = placeholder

    def _arrow(self) -> Text:
        return Text("> " if self.is_input else "  ", style=Style(color="blue"))

    def text(self) -> Text:
        if self.value:
            return (
                Text(self.value[: self.cursor_position], style=Style(color="white"))
                + Text("|", style=Style(color="blue"))
                + Text(self.value[self.cursor_position :], style=Style(color="white"))
            )
        return Text(self.placeholder, style=Style(color="#999999"))

    def render(self) -> Panel:
        return Panel(
            self._arrow() + self.text(),
            height=self.height,
            border_style="white",
            box=box.SQUARE,
        )

    def cursor_left(self):
        """光标左移"""
        if self.cursor_position > 0:
            self.cursor_position -= 1

    def cursor_right(self):
        """光标右移"""
        if self.cursor_position < len(self.value):
            self.cursor_position += 1

    def insert(self, v: str):
        if self.is_input:
            # 输入状态下
            if v == "escape":
                self.is_input = False
            elif v == "ctrl+h" and v:
                if self.cursor_position > 0:
                    self.value = f"{self.value[:self.cursor_position-1]}{self.value[self.cursor_position:]}"
                    self.cursor_left()
            elif v == "delete" and v:
                if self.cursor_position > 0:
                    self.value = f"{self.value[:self.cursor_position]}{self.value[self.cursor_position+1:]}"
            elif v == "left":
                self.cursor_left()
            elif v == "right":
                self.cursor_right()
            elif v == "home":
                self.cursor_position = 0
            elif v == "end":
                self.cursor_position = len(self.value)
            elif len(v) == 1:
                self.value = f"{self.value[:self.cursor_position]}{v}{self.value[self.cursor_position:]}"
                self.cursor_right()
            self.refresh()
        else:
            # 未输入状态下
            if v == "i":
                self.is_input = True

    def clear(self) -> str:
        """清空输入框同时返回输入框内容"""
        text: str = self.value
        self.value = ""
        self.cursor_position = 0
        self.refresh()
        return text



from .adapter import Adapter
from .terminal import print, console_view


async def shutdown():
    await console_view.shutdown()


async def start():
    await console_view.run()


__all__ = [
    "Adapter",
    "shutdown",
    "start",
    "print",
]



from rich import box
from rich.panel import Panel
from rich.style import Style
from textual.widget import Widget


class Right(Widget):
    def render(self):
        return Panel(
            "",
            box=box.SQUARE,
            # style=Style(bgcolor="white",)
        )



from typing import Type, Union, Mapping, Iterable, Optional

from rich.style import Style
from nonebot.typing import overrides
from rich.console import JustifyMethod

from nonebot.adapters import Message as BaseMessage
from nonebot.adapters import MessageSegment as BaseMessageSegment


class MessageSegment(BaseMessageSegment["Message"]):
    @classmethod
    @overrides(BaseMessageSegment)
    def get_message_class(cls) -> Type["Message"]:
        return Message

    @overrides(BaseMessageSegment)
    def __add__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return Message(self) + (
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    def __str__(self) -> str:
        return "".join(self.data.copy().get("text", ""))

    @overrides(BaseMessageSegment)
    def is_text(self) -> bool:
        return self.type == "text"

    @staticmethod
    def text(text: str) -> "MessageSegment":
        return MessageSegment("text", {"text": text})

    @staticmethod
    def markdown(
        markup: str,
        code_theme: str = "monokai",
        justify: Optional[JustifyMethod] = None,
        style: Union[str, Style] = "none",
        hyperlinks: bool = True,
        inline_code_lexer: Optional[str] = None,
        inline_code_theme: Optional[str] = None,
    ) -> "MessageSegment":
        return MessageSegment(
            "markdown",
            {
                "markdown": {
                    "markup": markup,
                    "code_theme": code_theme,
                    "justify": justify,
                    "style": style,
                    "hyperlinks": hyperlinks,
                    "inline_code_lexer": inline_code_lexer,
                    "inline_code_theme": inline_code_theme,
                }
            },
        )


class Message(BaseMessage[MessageSegment]):
    @classmethod
    @overrides(BaseMessage)
    def get_segment_class(cls) -> Type[MessageSegment]:
        return MessageSegment

    @overrides(BaseMessage)
    def __add__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return super(Message, self).__add__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @overrides(BaseMessage)
    def __radd__(
        self, other: Union[str, MessageSegment, Iterable[MessageSegment]]
    ) -> "Message":
        return super(Message, self).__radd__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @overrides(BaseMessage)
    def __iadd__(
        self, other: Union[str, MessageSegment, Iterable[MessageSegment]]
    ) -> "Message":
        return super().__iadd__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @staticmethod
    @overrides(BaseMessage)
    def _construct(
        msg: Union[str, Mapping, Iterable[Mapping]]
    ) -> Iterable[MessageSegment]:
        yield MessageSegment.text(msg)  # type: ignore

    @overrides(BaseMessage)
    def extract_plain_text(self) -> str:
        return "".join(seg.data["text"] for seg in self if seg.is_text())



from rich.text import Text
from rich.panel import Panel
from rich.style import Style
from rich.table import Table
from textual.widgets import Header
from textual.reactive import Reactive
from rich.console import RenderableType


class HeadBar(Header):
    """
    Custom Header for Gupshup showing status for the server and a Welcome message
    """

    status: Reactive[str] = Reactive("Console")

    def __init__(self):
        super().__init__(tall=False, style=Style(color="#b48ead", bgcolor="#3b4252"))

    def render(self) -> RenderableType:
        header_table: Table = Table.grid(padding=(1, 1), expand=True)
        header_table.style = self.style
        header_table.add_column(justify="left", ratio=0, width=20)
        header_table.add_column("title", justify="center", ratio=1)
        header_table.add_column("clock", justify="center", width=10)
        header_table.add_row(
            "  " + self.status,
            Text(
                "Welcome to Console Bot Test", style=Style(bold=True, color="#ea5252")
            ),
            self.get_clock() if self.clock else "",
        )
        header: RenderableType
        header = Panel(header_table, style=self.style) if self.tall else header_table
        return header



from asyncio import sleep
from datetime import datetime
from typing import Union, Optional

from rich.text import Text
from rich.tree import Tree
from rich.panel import Panel
from rich.style import Style
from nonebot.log import logger
from textual.widget import Widget
from rich.markdown import Markdown
from textual.geometry import Spacing
from textual.widgets import ScrollView

from ...message import Message
from ...config import BaseInfo, BotConfig


class ChatScreen(Widget):
    def __init__(self, name: Union[str, None] = None) -> None:
        super().__init__(name)
        self.name = "client"
        self.root = Tree(
            "🌲 [b green]Rich Tree", highlight=False, hide_root=True, expanded=True
        )
        self.node: Optional[Tree] = None
        self.sender: Optional[BaseInfo] = None
        self.padding = Spacing(0, 1, 0, 1)
        self.scroll: ScrollView = ScrollView(self, gutter=1)

    async def send_message(
        self, sender: BaseInfo, message: Union[str, Widget, Markdown]
    ) -> None:
        await sleep(0)
        if self.sender != sender:
            self.sender = sender
            logger.info(str(sender))
            self.node = self.root.add(
                Text(f"[{datetime.now().strftime('%H:%M:%S')}] ", style="#999999")
                + Text(f"{sender.nickname}", style=f"bold {sender.color}"),
                guide_style=Style(color="#1976d2"),
            )
        if self.node:
            self.node.add(message)
            self.refresh(layout=True)
            await self.scroll_end()

    async def scroll_end(self, refresh: bool = True) -> None:
        try:
            self.scroll.target_y = (
                self.scroll.window.virtual_size.height - self.scroll.size.height
            )
            self.scroll.animate("y", self.scroll.target_y, duration=0)
            if refresh:
                self.scroll.window.refresh(layout=True)
        except LookupError as err:
            ...

    def render(self):
        return self.root



from .app import ConsoleView

console_view: ConsoleView = ConsoleView()


def print(*args, seq: str = " ") -> None:
    console_view.logger.append(seq.join(str(v) for v in args))


__all__ = [
    "ConsoleView",
    "console_view",
    "print",
]



import os
from collections import defaultdict
from asyncio import wait, create_task, ensure_future
from typing import Dict, List, Union, Callable, Optional

from textual import events
from textual.app import App
from nonebot.log import logger
from rich.markdown import Markdown
from textual.widgets import ScrollView

from ..config import UserInfo
from .widgets.input import Input
from .widgets.right import Right
from .widgets.logger import Logger
from .widgets.header import HeadBar
from .widgets.client import ChatScreen
from ..event import Event, MessageEvent
from ..message import Message, MessageSegment


class ConsoleView(App):
    def __init__(self, user_info: Optional[UserInfo] = None) -> None:
        super().__init__()
        self.on: List[Callable] = []
        self.scroll_types: List[str] = []
        self.scroll_index: int = 0
        self.user_info: UserInfo = user_info or UserInfo(nickname="user", user_id="1")

    def trigger(self) -> None:
        create_task(wait(create_task(on()) for on in self.on))

    async def on_load(self) -> None:
        await self.bind("enter", "send_msg")
        await self.bind("up", "scroll_up")
        await self.bind("down", "scroll_down")

    async def action_swicth_scroll(self) -> None:
        """滚轮切换"""
        if self.scroll_index < len(self.scroll_types) - 1:
            self.scroll_index += 1
        else:
            self.scroll_index = 0

    async def _on(self, event: Event):
        if self.on:
            await wait([create_task(on(event)) for on in self.on])

    async def action_scroll_up(self) -> None:
        await self.scroll[self.scroll_types[self.scroll_index]].key_up()

    async def action_scroll_down(self) -> None:
        await self.scroll[self.scroll_types[self.scroll_index]].key_down()

    async def action_send_msg(self) -> None:
        if value := self.input.clear():
            event: MessageEvent = MessageEvent(
                user_info=self.user_info, message=Message(value)
            )
            await self.send_message(event, event.message)
            await self._on(event)

    async def send_message(
        self, event: MessageEvent, message: Union[str, Message, MessageSegment]
    ) -> None:
        """发送消息

        Args:
            event (MessageEvent): 消息事件
            message (Union[str, Message, MessageSegment]): 回复内容
        """
        try:
            if isinstance(message, MessageSegment):
                message = Message(message)
            elif isinstance(message, str):
                await self.client.send_message(event.user_info, message)

            if isinstance(message, Message) and message:
                widget = None
                text = None
                if md := message.get("markdown"):
                    widget = Markdown(**md[0].data["markdown"])
                else:
                    text = message.extract_plain_text()
                assert widget or text
                await self.client.send_message(event.user_info, widget or text or "")
        finally:
            logger.info(f"{event.user_info.nickname}: {message}")

    async def on_mount(self) -> None:
        self.width, self.height = os.get_terminal_size()
        self.head_bar: HeadBar = HeadBar()
        self.input: Input = Input()
        self.logger: Logger = Logger()
        self.client: ChatScreen = ChatScreen()
        self.scroll: Dict[str, ScrollView] = defaultdict(ScrollView)
        self.scroll_types = [self.client.name, self.logger.name]
        self.scroll[self.client.name] = self.client.scroll
        self.scroll[self.logger.name] = self.logger.scroll

        await self.view.dock(self.head_bar)
        await self.view.dock(
            self.scroll[self.logger.name],
            edge="right",
            name=self.logger.name,
            size=int(self.width * 0.39),
        )
        await self.view.dock(Right(), edge="right", size=2)
        await self.view.dock(
            self.scroll[self.client.name],
            name=self.client.name,
            size=self.height - self.input.height - 1,
        )
        await self.view.dock(self.input)

    async def on_key(self, event: events.Key) -> None:
        self.input.insert(event.key)
        if event.key == "escape":
            self.input.is_input = False
            self.head_bar.status = "No input"
        elif event.key == "i":
            self.input.is_input = True
            self.head_bar.status = "Input"
        elif event.key == "ctrl+i":
            await self.action_swicth_scroll()
            self.head_bar.status = f"{self.scroll_types[self.scroll_index]} scroll"

    async def run(self) -> None:
        await self.process_messages()



from typing import Union
from logging import StreamHandler

from rich.tree import Tree
from textual.widget import Widget
from textual.widgets import ScrollView
from nonebot.log import logger, default_filter, default_format


class ConsoleHandler(StreamHandler):
    def __init__(self, messages: "Logger") -> None:
        super().__init__(None)
        self.messages: "Logger" = messages

    def emit(self, record) -> None:
        self.messages.append(self.format(record))


class Logger(Widget):
    def __init__(self, name: Union[str, None] = None) -> None:
        name = name or "logger"
        super().__init__(name)
        self.root: Tree = Tree("", highlight=True, hide_root=True, expanded=True)
        self.handler: ConsoleHandler = ConsoleHandler(self)
        self.scroll: ScrollView = ScrollView(self, gutter=1)
        logger.remove()
        logger.add(
            self.handler,
            filter=default_filter,
            format=default_format,
        )

    def scroll_end(self, refresh: bool = True) -> None:
        self.scroll.target_y = (
            self.scroll.window.virtual_size.height - self.scroll.size.height
        )
        self.scroll.animate("y", self.scroll.target_y, duration=0)
        if refresh:
            self.scroll.window.refresh(layout=True)

    def append(self, msg: str) -> None:
        self.root.add(msg)
        self.refresh(layout=True)
        self.scroll_end()

    def render(self) -> Tree:
        return self.root



import os
from typing import Any

from nonebot.drivers import Driver
from nonebot.typing import overrides

from nonebot.adapters import Adapter as BaseAdapter

from .bot import Bot
from .event import Event
from .config import BotConfig
from .terminal import console_view


class Adapter(BaseAdapter):
    @overrides(BaseAdapter)
    def __init__(self, driver: Driver, **kwargs: Any) -> None:
        super().__init__(driver, **kwargs)
        self.bot = Bot(self, BotConfig(user_id="0"))

        @console_view.on.append
        async def _handle_event(event: Event) -> None:
            await self.bot.handle_event(event)

        self.setup()

    @staticmethod
    @overrides(BaseAdapter)
    def get_name() -> str:
        return "Console"

    @overrides(BaseAdapter)
    async def _call_api(self, bot: Bot, api: str, **data: Any) -> Any:
        if api == "send_message":
            await console_view.client.send_message(bot.bot_config, **data)

    def setup(self):
        @self.driver.on_startup
        async def _start() -> None:
            self.bot_connect(self.bot)
            await console_view.run()
            # os.system("exit")
            # os.system("exit")

        @self.driver.on_shutdown
        async def _stop() -> None:
            await console_view.shutdown()
            self.bot_disconnect(self.bot)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot',
 'nonebot.adapters.console',
 'nonebot.adapters.console.terminal',
 'nonebot.adapters.console.terminal.widgets']

package_data = \
{'': ['*']}

install_requires = \
['nonebot2>=2.0.0-beta.1,<3.0.0', 'textual==0.1.18']

setup_kwargs = {
    'name': 'nonebot-adapter-console',
    'version': '0.2.0',
    'description': 'console adapter for nonebot2',
    'long_description': 'None',
    'author': 'MelodyKnit',
    'author_email': 'yanximelody@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.0,<4.0.0',
}


setup(**setup_kwargs)



from pydantic import BaseModel


class BaseInfo(BaseModel):
    nickname: str
    user_id: str
    color: str = "green"


class BotConfig(BaseInfo):
    nickname: str = "TestBot"
    color: str = "blue"


class UserInfo(BaseInfo):
    ...
